package prupp;

// $Id: Configurer.java,v 1.2 1998/12/19 13:46:09 d94-rol Exp $

import java.io.*;
import java.net.*;
import java.lang.reflect.*;
import java.util.*;

/** A MetaPage to handle runtime configuration of filters.
 */
public class Configurer implements MetaHandler {
    
    /** default constructor */
    public Configurer () {
    }
    
    /** handler of a MetaPage. 
     * @param input the InputStream from the client.
     * @param output the OutputStream to the client.
     * @param url the requested url.
     * @param htab the supplied argument to the page (CGI-parameters).
     * @param uap the username and paswword as by Basic-authentication
     */
    public void handle (DataInputStream input, DataOutputStream output, URL url, Properties htab, String uap) {
	String setupError = null;
	String error = null;
	String name = htab.getProperty ("configureclass");
	if (name == null)
	    name = "";
	String ok = htab.getProperty ("APPLY");
	if (!name.equals ("") && ok != null)
	    setupError = setup (name, htab);
	// TODO this should really be a select.
	StringBuffer res = new StringBuffer (Proxy.getHeader ().toString () +
					     HTMLPage.getPageHeader ("Configurer") +
					     "<form name=FilterConfig action=\"http://proxy/" + getClass ().getName () + "\" method=get>\n" + 
					     "Select the class you want to configure<br>" + 
					     "<B>Class:</B> <input type=\"textfield\" size=\"30\" name=\"configureclass\" value=\"" + name + "\">\n" +
					     HTMLPage.getTableHeader (100, 1) +
					     HTMLPage.getTableTopicRow () + 
					     "\t<th>field</th><th>current value</th></tr>\n");

	if (!name.equals ("")) {
	    try {
		if (name.indexOf (".") < 0)
		    name = "prupp." + name;
		Class cls = Class.forName (name);  // throws exception unless found.
		Class params[] = new Class[0];
		Method setup = cls.getDeclaredMethod ("getProperties", params);
		Object arguments[] = new Object [0];
		Properties prop = (Properties)setup.invoke (cls, arguments);	
		Enumeration e = prop.propertyNames ();
		while (e.hasMoreElements ()) {
		    String pname = (String)e.nextElement ();
		    String value = prop.getProperty (pname);	
		    res.append ("\t<tr><td>" + pname + "</td><td><center><input type=\"textfield\" size=\"60\" name=\"" + pname + "\" value=\"" + value + "\"></center></td></tr>\n");
		}
	    } catch (ClassNotFoundException e) {
		error = "Couldnt find: " + name;
	    } catch (NoSuchMethodException e) {
		error = "Couldnt find method getProperties for: " + name + " (guess it aint configurable)";
	    } catch (SecurityException e) {
		error = "Seems reflection aint allowed.";	      
	    } catch (InvocationTargetException e) {
		error = "couldnt invoke getProperties for " + name + 
		    ", nested exception is:" + e.getTargetException ();	
	    } catch (IllegalAccessException e) {
		error = e.toString ();
	    }
	    if (error != null)
		Proxy.logError (error);
	} else {
	    error = "No class to configure";
	}
	res.append ((setupError == null ? "" : "\t<tr><td colspan=2 bgcolor=\"#ff7777\"><font color=\"black\">Error:"+setupError+"</font></td></tr>\n") +
		    (error == null ? "" : "\t<tr><td colspan=2 bgcolor=\"#ff7777\"><font color=\"black\">Error:"+error+"</font></td></tr>\n") +
		    "<tr><td colspan=2 align=center>" +
		    "Using [APPLY] sets the values for now <!--but doesnt write the settings back to disk--><br>\n" +
		    "<!--Using [SAVE] does everything apply does and also makes sure the settings are written back to disk<br-->\n"+
		    "<input type=submit value=\"APPLY\" name=\"APPLY\">" +
		    "<!--input type=submit value=\"SAVE\" name=\"SAVE\"-->" +
		    "</td></tr>\n");
	res.append ("</table></form>\n</body></html>");
	try {      
	    output.write (res.toString ().getBytes ());
	} catch (IOException e) {
	    Proxy.logError ("Couldnt write FilterConfig");
	}
    }

    private String setup (String name, Properties prop) {
	String error = null;
	try {
	    if (name.indexOf (".") < 0)
		name = "prupp." + name;
	    Class cls = Class.forName (name);  // throws exception unless found.
       
	    Class params[] = new Class[1];
	    params[0] = Properties.class;
	    Method setup = cls.getDeclaredMethod ("setup", params);
      
	    Object arguments[] = new Object [1];
	    arguments[0] = prop;
	    setup.invoke (cls, arguments);	      
      
	} catch (ClassNotFoundException e) {
	    error = "Couldnt find " + name;
	} catch (NoSuchMethodException e) {
	    error = "couldnt find method setup for: " + name + " (this is probably nothing to worry about)";
	} catch (SecurityException e) {
	    error = "Seems reflection aint allowed.";
	} catch (InvocationTargetException e) {
	    error = "couldnt invoke setup for " + name + 
		", nested exception is:" + e.getTargetException ();
	} catch (IllegalAccessException e) {
	    error = e.toString ();
	}
	if (error != null) {
	    error = "Setup failed: " + error;
	    Proxy.logError (error);
	}    
	return error;
    }
}
