package prupp;

// $Id: ImageSender.java,v 1.7 1999/01/17 22:01:42 d94-rol Exp $

import java.util.*;
import java.io.*;
import java.net.*;

/** This class sends an image to the assigned outputstreams.
 */
public class ImageSender
{
    private long origsize = -1;
    private long lowqualitysize = -1;

    /** The standard constructor.
     * @param os the standard stream to send to.
     * @param cache the cachefile if any (null = no cache).
     * @param url the URL were writing from
     */
    public ImageSender (DataOutputStream os, DataInputStream webis, HTTPHeader hd, NCacheEntry ent) {
	DataOutputStream file = null;
	DataOutputStream cache = null;
	byte v[] = new byte[4096];
	int i;

	if (ent != null) {
	    try {
		file = new DataOutputStream (new FileOutputStream(ent.getFileName ()));
	    } catch (IOException e){
		Proxy.logError ("Couldnt find the file " + ent.getFileName ());
		return;
	    } 
	}	    
	else 
	    Proxy.logError ("File not cacheable, wont convert!");
      
	//it is probably stupid to convert really small files
	String cl = hd.getHeader ("Content-length");
	if (file != null && cl != null && Integer.parseInt (cl) > 2000 && Proxy.getConvert () != null) {
	    try {

		// read in the file. (over a speedy link).
		while ((i = webis.read(v)) > 0)
		    file.write (v, 0, i);
		file.close ();
		origsize = new File (ent.getFileName ()).length ();
		
	    } catch (IOException e){
		Proxy.logError ("Error while reading image");
	    }
	    
	    try {
		// convert it.
		String[] args = {Proxy.getConvert (), 
				 "-quality", 
				 "10",                        // 10% quality is actually good :-)
				 ent.getFileName () + "[0]",  // remove any gif-animations etc.
				 "jpeg:"+ent.getFileName()+".c" };
		//System.out.println ("will now convert " +  ent.getFileName ());
		Process ps = Runtime.getRuntime ().exec (args);
		ps.waitFor ();
		if (ps.exitValue () != 0) 
		    Proxy.logError ("Bad conversion " + ent.getFileName ());
	    } catch (IOException e) {
		Proxy.logError ("Conversion failed: " + e);
	    } catch (InterruptedException e) {
		Proxy.logError ("interupting my sleep?");
	    }
	    File convertedFile = new File (ent.getFileName () + ".c");
	    lowqualitysize = convertedFile.length ();
	    if (origsize < lowqualitysize) {
		File orig = new File (ent.getFileName ());
		orig.renameTo (convertedFile);
	    }
	    DataInputStream imfile = null; 
	    try {
		imfile = new DataInputStream (new FileInputStream (convertedFile));
		file = new DataOutputStream (new FileOutputStream (ent.getFileName ()));

		hd.setHeader ("content-type", "image/jpeg");
		hd.removeHeader ("content-length");	      // if we supply it we cant lie, so be safe.
		byte hdbytes[] = hd.toString (). getBytes ();    
		os.write (hdbytes);
		file.write (hdbytes);
		    
		// send converted file.
		while ((i = imfile.read (v)) > 0) {
		    file.write (v, 0, i);
		    os.write (v, 0, i);
		}	    
		file.close ();
		imfile.close ();
	    } catch (IOException e) {
		// most often due to a broken pipe on the client...
		try {
		    if (file != null)
			file.close ();
		    if (imfile != null)
			imfile.close ();	  
		} catch (IOException e2) {} // close and ignore..
		Proxy.logError ("Couldn't read converted file(" + ent.getFileName () + ".c):" + e);	    
	    }
	    convertedFile.delete ();
	}
	else {  // small file or image we wont filter.
	    try {
		byte hdbytes[] = hd.toString (). getBytes ();
		os.write (hdbytes);
		if (file != null)
		    file.write (hdbytes);
	  
		while ((i = webis.read(v)) > 0) {
		    os.write (v, 0, i);
		    if (file != null)
			file.write (v, 0, i);
		}
		file.flush();
		os.flush();
		if (file != null)
		    file.close ();
	    } catch (IOException e) {
		Proxy.logError ("Couldn't handle small file (" + e + ") URL: " + ent.getKey () + ", File: " + ent.getFileName ());	
		//e.printStackTrace ();
		try {
		    if (file != null)
			file.close ();
		} catch (IOException e2) {}
	    }
	}
    }
    
    /** get the originalsize of the image 
     * @return the original filesize of the image.
     */
    public long getOrigSize () {
	return origsize;
    }
    
    /** get the size of the converted image 
     * @return the filesize of the converedimage.
     */
    public long getLowqualitySize () {
	return lowqualitysize;
    }

    /** get the ratio of compreesion for the image.
     * @return lowqualitysize / origsize
     */
    public float getRatio () {
	if (origsize != 0)
	    return ((float)lowqualitysize / (float)origsize);
	return 1;
    }
}
