package prupp;

// $Id: MetaPageHandler.java,v 1.10 1998/12/19 13:46:09 d94-rol Exp $

import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.util.*;

/** The proxys MetaPageHandler.
 *  To add MetaPage:s to the proxy just add the needed classes.
 *  Reflection is used to look up meta-pages.
 */
public class MetaPageHandler extends Responder {
  
    /** basic constructor
     * @param socket The Socket to the client.
     * @param reqLine The request being handled.
     * @param input The inputstream from the client.
     * @param output The outputstream to the client.
     * @param url the URL of the request.
     * @param header the full HTTP header for the request.
     */
    public MetaPageHandler (Socket socket, String reqLine, DataInputStream input, DataOutputStream output, URL url, HTTPHeader header) {
	super (reqLine, input, output);
	if (clientis == null || clientos == null || url == null || header == null) {
	    Proxy.logError (getClass ().getName () + " called with null argument(s)");
	    return;
	}
    
	String uap = header.getHeader ("Proxy-Authorization");

	if (Proxy.isMetaRestricted ()) {
	    if (uap == null) {
		requestAuthentication ("proxy", url);
		return;
	    } else {
		if (uap.startsWith ("Basic ")) {
		    uap = uap.substring ("Basic ".length ());
		} else {  // unknown method?
		    requestAuthentication ("proxy", url);
		    return;
		}
		String userapass = Util.uudecode (uap);
		int i = -1;
		if ((i = userapass.indexOf (":")) > -1) {
		    String userid = userapass.substring (0, i);
		    String pass = userapass.substring (i+1);
		    setAuthuser (userid);
		    if (!Proxy.checkBasicAccess (userid, pass)) {
			requestAuthentication ("proxy", url);
			return;
		    }
		} else {
		    requestAuthentication ("proxy", url);
		    return;
		}	
	    }  
	}
	
	String file = url.getFile ().substring (1);  // remove initial '/'

	int index = -1;
	String args = "";
	if ((index = file.indexOf ("?")) >= 0) {
	    args = file.substring (index + 1);
	    file = file.substring (0, index);
	}
	Properties htab = splitArgs (args);
    
	if ((index = file.indexOf ("/")) >= 0) {
	    String fc = file.substring (index + 1);
	    file = file.substring (0, index);
	    htab.put ("argstring", fc);
	}

	String error = null;
	try {
	    if (file.indexOf (".") < 0)
		file = "prupp." + file;

	    Class cls = Class.forName (file);
	    MetaHandler mh = (MetaHandler)cls.newInstance ();
	    mh.handle (clientis, clientos, url, htab, uap);

	    // Now take care of every error...
	} catch (NoSuchMethodError e) {
	    error = "Given metahandler doesnt have a public no-arg constructor:" + file + ", " + e;
	} catch (ClassCastException e) {
	    error = "Given metapage is not a MetaHandler:" + file + ", " + e;
	} catch (ClassNotFoundException e) {
	    error = "Couldnt find class:" + file + ", " + e;
	} catch (InstantiationException e) {
	    error = "Couldnt instantiate metahandler:" + file + ", " + e;
	} catch (IllegalAccessException e) {
	    error = "Que? metahandler access violation?:" + file + ", " + e;
	} catch (IllegalArgumentException e) {
	    error = "Strange name of metapage?:" + file + ", " + e;
	} 
	if (error != null) {
	    setStatusCode (400);
	    Proxy.logError (error);
	    error (error);
	    return;
	}
    }

    /** splits the CGI-paramsstring into variables and values.
     *  put these values into a hashtable for easy retrival
     * @param params the CGI-querystring.
     * @return a hastable with type->value maps for the CGI-querystring
     */
    protected Properties splitArgs (String params) {
	Properties htab = new Properties ();
	StringTokenizer st = new StringTokenizer (params, "=&", true);
	String key = null;
	while (st.hasMoreTokens ()) {
	    String next = st.nextToken ();
	    if (next.equals ("=")) {
		// nah..
	    } else if (next.equals ("&")) {
		if (key != null) {
		    htab.put (key, "");
		    key = null;
		}
	    } else if (key == null) {
		key = next;
	    } else {
		htab.put (key, Util.URLdecode (next));
		key = null;
	    }
	}
	return htab;
    }
}


