package prupp;

// $Id: Proxy.java,v 1.21 1999/01/17 22:01:42 d94-rol Exp $
// $Author: d94-rol $
// $Date: 1999/01/17 22:01:42 $

import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.reflect.*;
import java.text.*;

/** This is the central dispatcher for RabbIt,
 *  get a connection and create a HTTPHeaderHandler for the connection.
 *  Also keeps a lot of variables.
 *  Handles logging so it goes out nicely formatted.
 */
public class Proxy extends Thread {
    /** current version */
    public static final String VERSION = "RabbIT proxy version 1.8";
    /** which file contains help? */
    public static final String HELPFILE = "Help.txt";
    /** the standard port */
    protected static final int PORT = 9666;
    /** the standard inifile */
    protected static String inifile = "conf/proxy.conf";
  
    /** the current connections */
    private static Vector convec;
    /** the time the proxy started */
    private static Date started;
    /** a map somehost -> ipnumber */
    private static Hashtable ips = new Hashtable ();    

    /** the current filters */
    private static Vector filterVec;

    /** total number of connections handled */
    private static int total = 0;
    /** maximum concurrent connections */
    private static int maxconnections = 50;
    /** have a gui? (only usefull when debugging) */
    private static boolean gui = false;

    /** the log of things */
    static Logger log = null;
    /** the logview (only in -gui mode)*/
    static LogView lv = null;

    /** the port were listening on */
    private static int port = PORT;
    /** The host we run on. */
    private static InetAddress localhost;
    private ServerSocket ss;
    private ThreadGroup tg;

    private static String configfile;
    /** the config loaded from the inifile */
    private static Properties config = new Properties ();
    /** output for errors */
    private static PrintWriter errorlog;
    /** output for accesses */
    private static PrintWriter accesslog;

    /** adress of connected proxy */
    private static InetAddress proxy = null;
    /** port of connected proxy */
    private static int proxyport = -1;
    /** where we find convert */
    private static String convert = null;
    /** the cache-handler */
    private static NCache cache = new NCache ();
    /** small logo for meta-pages etc.*/
    private static String smalllogo =
    "http://www.student.nada.kth.se/~d94-rol/pics/RabbIT.gif";    // default if none specified in config.

    /** the currently loaded users (userid->password (in cleartext) */  // password should probalby be encrypted.
    private static Hashtable users;
  
    /** where we store the allowed ranges of ips */
    private static Vector allowed;
  
    /** the current running proxy */
    private static Proxy instance;
  
    /** The format we write dates on. */
    private static SimpleDateFormat sdf = new SimpleDateFormat ("dd/MMM/yyyy:HH:mm:ss zz");

    /** Start a proxy. 
     *  Parse flags and read the config, then starts the proxy.
     * @param args the command-line flags given.
     */ 
    public static void main (String[] args) {
	int p = PORT;
	int i = 0;
	InetAddress proxy = null;
	int proxyport = -1;

	// Take care of arguments given.
	// NOTE most args should be in the config-file.
	while (args.length > i) {
	
	    // use custom ini-file?
	    if (args[i].equals ("-f")) {
		i++;
		if (args.length > i) {
		    inifile = args[i];
		}
		else {
		    System.err.println ("No file specified");
		    System.exit (-1);
		}
	    }
	
	    // version ?
	    else if (args[i].equals ("-v")) {
		System.out.println (VERSION);
		System.exit (0);
	    }	

	    // user wants help??
	    else if (args[i].equals ("-?") || args[i].equals("-h") || args[i].equals ("--help")) {
		printHelp();
		System.exit (0);
	    }

	    // use a gui?
	    else if (args[i].equals ("-gui")) {
		gui = true;
	    }
	    else {
		System.out.println ("Unsupported argument " + args[i]);	 
	    }
	    i++;
	}
      
	readIniFile (inifile);
	
	instance = new Proxy ();
    }
    
    /** Constructor, only available from here.
     *  @param config the Property that have all configurable switches.
     */    
    private Proxy () {
	super ("Rabbit proxy");
      
	started = new Date ();
	setup (config);
	
	this.port = port;
	try {
	    ss = new ServerSocket (port);
	}
	catch (IOException e) {
	    System.err.println ("Failed to open serversocket on port " + port);
	    System.exit (1);
	}
	tg = new ThreadGroup ("Rabbit proxy connections");
	convec = new Vector ();
	    
	log = new Logger ();
	if (gui) {
	    lv = new LogView (log);
	}
	
	try {
	    localhost = InetAddress.getLocalHost ();
	} catch (UnknownHostException e) {
	    System.err.println ("I couldnt find the hostname of this machine, exiting.");
	    System.exit (-1);
	}

	start ();
    }

    /** while we can, accept new sockets, creating a handler for each
     *  and also save connections for statistics.
     */
    public void run () {
	logError ("Started");
	while (true) {
	    Socket s = null;

	    // Not to many at once.
	    while (convec.size () > maxconnections) 
		yield ();   // busy-loop :-/
	    try {
		s = ss.accept ();
		s.setSoTimeout (120*1000);             // 2 min timeout on read, We should be "near" the client.
		total++;
		convec.addElement (new HTTPHeaderHandler (s));
	    } catch (SocketException e) {
		Proxy.logError ("Socketoptions failed?: " + e);
		try {
		    if (s != null)
			s.close();
		} catch (IOException ee) {}
	    } catch (IOException e) {
		Proxy.logError ("failed to accept, trying to restart.");
		try {
		    ss.close ();
		} catch (IOException e2) {
		    Proxy.logError ("Couldnt close serversocket, shutting down");
		    System.exit (-1);
		}
		try {
		    ss = new ServerSocket (port);
		} catch (IOException e2) {
		    Proxy.logError ("Couldnt open serversocket, shutting down");
		    System.exit (-1);
		}
	    } catch (Exception e) {
		Proxy.logError ("Unknown error: " + e);
	    }
	}
    }

    /** test if the socket that connects is a ok one 
     * @param socket the Socket that connects
     */
    public static boolean allowed (Socket s, int type) {
	InetAddress ia = s.getInetAddress ();
	Enumeration e = allowed.elements ();
	while (e.hasMoreElements ()) {
	    Access acc = (Access) e.nextElement ();
	    if (acc.inrange (s.getInetAddress ()) && ((acc.getType () & type) != 0))
		return true;
	}
	return false;
    }
    
    /** Remove a connection (it has received full treatment)
     *  @param con the connection that is closing down.
     */
    public static synchronized void removeConnection (HTTPHeaderHandler con) {
	convec.removeElement (con);
    }

    /** log accesses.
     * @param socket the Socket were talking to.
     * @param responder the responsehandler that handled the request.
     */
    public static synchronized void logRequest (Socket socket, Responder responder) {
	accesslog.println (socket.getInetAddress ().getHostAddress().toString() + 
			   " - " +   // dont think Ill ever check up identd
			   responder.getAuthuser () + " " + 
			   sdf.format(new Date ()) + 
			   " \"" + responder.getRequestLine () + "\" " + 
			   responder.getStatusCode () + 
			   " - " +
			   responder.getExtraInfo ());
    }
  
    /** log errors and other important stuff. 
     * @param error the thing that happend.
     */
    public static synchronized void logError (String error) {
	errorlog.println ("[" + sdf.format (new Date ()) + "][" + error + "]");
    }
  

    /** gimme the filters for the object o.
     * @param o the object were filtering.
     * @return a Vector of Filters
     */ 
    public static Vector getFilters (Object o) {
	Vector fl = new Vector ();
	Class args[] = new Class [1];
	args [0] = o.getClass ();
	Object cargs[] = new Object[1];
	cargs[0] = o;
	try {	
	    // try to get the correct constructor.
	    for (int i = 0; i < filterVec.size (); i++) {
		Class c = (Class)filterVec.elementAt(i);
		Constructor[] cons= c.getConstructors ();
		Constructor con = null;
		try {
		    con = c.getConstructor (args);
		} catch (Exception e) {
		}
		if (con != null) 
		    fl.addElement (con.newInstance (cargs));
		else 
		    fl.addElement (c.newInstance ());
	    }
	} catch (Exception e) {
	    Proxy.logError ("Couldnt get a filter constructor !?!:" + e);
	}
	return fl;
    }        

    /** get the inetAdress of an URL
     * @param urltoget the URL were intrested in.
     * @return an inetadress of null on error.
     */
    public static InetAddress getInetAddress (URL urltoget) throws UnknownHostException {
	if (proxy != null)
	    return proxy;
	InetAddress ia = (InetAddress)ips.get (urltoget.getHost ());
	if (ia == null) {
	    ia = InetAddress.getByName(urltoget.getHost());
	    ips.put (urltoget.getHost (), ia);
	}
	return ia;
    }

    /** Get the port to connect to.
     * @param port the port we want to connect to.
     * @return the port to connect to.
     */
    public static int getConnectPort (int port) {
	if (proxy != null)    // are we talking through another proxy?
	    return proxyport;
	return port;
    }

    /** are we connected to another proxy? 
     * @return true if the proxy is connected to another proxy.
     */
    public static boolean isProxyConnected () {
	return proxy != null;
    }

    /** Get the Log-handle
     * @return the Logger we use.
     */ 
    static Logger getLogger () {
	return log;
    }

    /** print out the helptext to the user.
     */
    public static void printHelp () {
	try {
	    InputStream f = new FileInputStream (HELPFILE);
	    byte[] b = new byte[4096];
	    int i;
	    while ((i = f.read(b)) > 0) 
		System.out.write (b, 0, i);
	    f.close ();
	} catch (IOException e) {
	    System.err.println ("Couldnt read helptext");
	}
    }

    /** load the config for the Proxy. 
     */
    protected static void readIniFile (String filename) {
	try {
	    InputStream f = new FileInputStream (filename);
	    configfile = filename;
	    config = new Properties ();
	    config.load (f);
	    f.close ();
	    //config.list (System.out);
	} catch (IOException e) {
	    System.err.println ("Couldnt read configfile: " + filename);
	    System.exit (-1);
	}
    }

    /** set the path to convert, the program we use to convert images
     * @param conv the new path to convert
     */
    public static void setConvert (String conv) {
	File f = new File (conv);
	if (!f.exists () || !f.isFile()) {
	    Proxy.logError ("convert (" + conv + ") not found, is your path correct?");
	    conv = null;
	}
	convert = conv;
    }

    /** get the path to convert
     * @return the full path to convert
     */
    public static String getConvert () {
	return convert;
    }

    /** Get the small logo string.
     * @return the string containing the url to the small logo
     */
    public static String getSmallLogo () {
	return smalllogo;
    }
  
    /** Set the small logo used on meta-pages etc
     * @param s the new string to the logo 
     */
    public static void setSmallLogo (String s) {
	smalllogo = s;
    }
  
    /** handle the setup of this Proxy.
     *  Should be nice and cope with runtime setup.
     *  NOTE! the proxy will die if it is setup wrong right now, this needs
     *  a fix since runtimeconfig is available and then you only want an 
     *  errormessage.
     *  NOTE2! The proxy needs to be restarted for some settings to take place
     *  this is not done!
     * @param config the Property to use.
     */
    protected static void setup (Properties config) {    
	// fatal errors here and we exit. Not much else to do.
    
	// set up errorlog first so we can log things correctly..
	String elog = config.getProperty ("errorlog");
	try {
	    if (elog != null) {
		if (errorlog != null) {
		    errorlog.flush ();
		    errorlog.close ();
		}
		if (!elog.equals ("")) 
		    errorlog = new PrintWriter (new FileWriter (elog, true),true);       
		else 
		    errorlog = new PrintWriter (System.err);
	    } else {
		if (errorlog == null){
		    elog = "";
		    errorlog = new PrintWriter (System.err);
		}
	    }
	} catch (IOException e) {
	    System.err.println ("couldnt create errorlog: " + elog);
	    System.exit (-1);
	}

	String alog = config.getProperty ("accesslog");
	try {
	    if (alog != null) {
		if (accesslog != null) {
		    accesslog.flush ();
		    accesslog.close ();
		}
		if (!alog.equals ("")) 
		    accesslog = new PrintWriter (new FileWriter (alog, true),true);       
		else 
		    accesslog = new PrintWriter (System.err);
	    } else {
		if (accesslog == null){
		    alog = "";
		    accesslog = new PrintWriter (System.err);
		}
	    }
	} catch (IOException e) {
	    System.err.println ("couldnt create accesslog: " + alog);
	    System.exit (-1);
	}

	String p = config.getProperty ("port");
	if (p != null) {
	    try {
		port = Integer.parseInt (p);
		if (port < 1024) {
		    logError ("port must be over 1024 since I dont want to run as root");
		    System.exit (-1);
		}
		Proxy.config.put ("port", p);
	    } catch (NumberFormatException e) {
		logError ("Strange port: " + p);
		System.exit (-1);
	    }
	}
    
	Proxy.config.put ("proxyhost", "");
	Proxy.config.put ("proxyport", "");
	proxy = null;
	String pname = config.getProperty ("proxyhost");
	String pport = config.getProperty ("proxyport");
	if (pname != null && !pname.equals ("") && pport != null && !pport.equals ("")) {
	    Proxy.config.put ("proxyhost", pname);
	    Proxy.config.put ("proxyport", pport);
	    try {
		proxy = InetAddress.getByName (pname);
	    } catch (UnknownHostException e) {
		logError ("Unknown proxyhost: " + pname);
		System.exit (-1);
	    }
	    try {
		proxyport = Integer.parseInt (pport);	
	    } catch (NumberFormatException e) {
		logError ("Strange proxyport: " + pport);
		System.exit (-1);
	    }
	}
	else if ((pname != null && pport == null) || (pname == null && pport != null)) {
	    logError ("You must specify both proxyhost and proxyport");
	    System.exit (-1);
	} 

	String cdir = config.getProperty ("cachedir");
	if (cdir != null) {
	    cache.setCacheDir (cdir);
	} else {
	    cdir = cache.getCacheDir ();
	}
	Proxy.config.put ("cachedir", cdir);

	String cmsize = config.getProperty ("cachemaxsize");
	if (cmsize != null && !cmsize.equals ("")) {
	    try {
		cache.setMaxSize (Integer.parseInt (cmsize) * 1024 * 1024); // in MB..
	    } catch (NumberFormatException e) {
	    }
	} else 
	    cmsize = "" + (cache.getMaxSize () / (1024*1024));
	Proxy.config.put ("cachemaxsize", cmsize);
	
	String ctime = config.getProperty ("cachetime");
	if (ctime != null && !ctime.equals ("")) {
	    try {
		cache.setCacheTime (Integer.parseInt (ctime) * 1000 * 60 * 60); // in Hours..
	    } catch (NumberFormatException e) {
	    }
	} else 
	    ctime = "" + (cache.getCacheTime ()/(1000*60*60));
	Proxy.config.put ("cachetime", ctime);
    
	String conv = config.getProperty ("convert");
	if (conv != null && !conv.equals ("")) {
	    setConvert (conv);
	} else if ((conv = Proxy.config.getProperty ("convert")) == null){
	    logError ("convert not found, imageprocessing disabled.");
	    conv = "";
	    convert = null;
	} 
	Proxy.config.put ("convert", conv);
	
	String slogo = config.getProperty ("smalllogo");
	if (slogo != null) {
	    smalllogo = slogo;
	} else if ((slogo = Proxy.config.getProperty ("smalllogo")) == null)
	    slogo = smalllogo;
	Proxy.config.put ("smalllogo", slogo);
    
	String mc = config.getProperty ("maxconnections");
	if (mc != null) {
	    try {
		maxconnections=Integer.parseInt (mc);
	    } catch (NumberFormatException e) {
		logError ("bad number for maxconnections: " + mc + ", using old value: " + maxconnections);
		mc = "50";
	    }
	} else if ((mc = Proxy.config.getProperty ("maxconnections")) == null) {
	    mc = "50";
	    maxconnections = 50;
	}
	Proxy.config.put ("maxconnections", mc);

	String filterconfig = config.getProperty ("filterconfig");
	if (filterconfig != null) {
	    if (!filterconfig.equals("")) 
		readFilterConfig ();
	    else 
		filterconfig = "";       
	} else 
	    filterconfig = Proxy.config.getProperty ("filterconfig");
	Proxy.config.put ("filterconfig", filterconfig);

	String userfile = config.getProperty ("basicuserfile");
	if (userfile != null) {
	    if (!userfile.equals("")) 
		readUsers ();
	    else 
		userfile = "";
	} else 
	    userfile = Proxy.config.getProperty  ("basicuserfile");
	Proxy.config.put ("basicuserfile", userfile);
	
	String accessfile = config.getProperty ("access");
	if (accessfile != null) {
	    if (!accessfile.equals("")) 
		readAccess ();
	    else 
		accessfile = "";
	} else 
	    accessfile = Proxy.config.getProperty ("access");
	Proxy.config.put ("access", accessfile);

	String rmeta = config.getProperty ("restrictMeta");
	if (rmeta != null) 
	    Proxy.config.put ("restrictMeta", rmeta);
	else if (Proxy.config.getProperty ("restrictMeta") == null)
	    Proxy.config.put ("restrictMeta", "true");

	String name = config.getProperty ("name");
	if (name == null) {
	    name = Proxy.config.getProperty ("name", "RabbIT");
	}
	Proxy.config.put ("name", name);
    
	HTMLPage.setup ();
    }

    /** save the config back to file
     */
    public static synchronized void saveConfig (){
	try {
	    OutputStream pw = new FileOutputStream (configfile);
	    Proxy.config.save (pw, "Automatically generated");
	    pw.flush ();
	    pw.close ();
	} catch (IOException e) {
	    
	}
    }
    
    /** save filter configuration back to file
     */
    public static synchronized void saveFilterConfig () {
	Config cfg = new Config ();
	String filters = "";
	String error = null;
	try {
	    OutputStream pw = new FileOutputStream (Proxy.config.getProperty ("filterconfig", "conf/filter.conf"));
	    for (int i = 0; i < filterVec.size (); i++) {
		Class cl = (Class)filterVec.elementAt (i);
		Properties prop = new Properties ();
		try {
		    Class params[] = new Class[0];
		    filters += cl.getName () + ",";
		    Method setup = cl.getDeclaredMethod ("getProperties", params);
		    Object arguments[] = new Object [0];
		    prop = (Properties)setup.invoke (cl, arguments);	
		} catch (NoSuchMethodException e) {
		    error = "Couldnt find method getProperties for: " + cl + " (guess it aint configurable)";
		} catch (SecurityException e) {
		    error = "Seems reflection aint allowed.";	      
		} catch (InvocationTargetException e) {
		    error = "couldnt invoke getProperties for " + cl + 
			", nested exception is:" + e.getTargetException ();	
		} catch (IllegalAccessException e) {
		    error = e.toString ();
		}
/*		System.out.println ("setting properties for: " + cl);
		prop.save (System.out, cl.toString ());*/
		cfg.setProperties (cl.getName (), prop);
	    }
	    filters = filters.substring (0, filters.length () - 1);
	    cfg.setProperty ("", "filters", filters);
	    cfg.save (pw, "Automatically genrated");
	    pw.flush ();
	    pw.close ();
	} catch (IOException e) {
	}
	if (error != null) 
	    Proxy.logError ("couldnt save filterconfig");
    }

    /** save access configuration back to file
     */
    public static synchronized void saveAccess () {
	String error = null;
	try {
	    PrintWriter pw = new PrintWriter (new FileOutputStream (Proxy.config.getProperty ("access", "conf/access.conf")));
	    Enumeration e = allowed.elements ();
	    while (e.hasMoreElements ()) {
		Access acc = (Access) e.nextElement ();
		pw.println (acc.toString ());
	    }
	    pw.flush ();
	    pw.close ();
	} catch (IOException e) {
	}
	if (error != null) 
	    Proxy.logError ("couldnt save accessconf");
    }

    /** save (auth)user configuration back to file
     */
    public static synchronized void saveUsers () {
	String error = null;
	try {
	    PrintWriter pw = new PrintWriter (new FileOutputStream (Proxy.config.getProperty ("basicuserfile", "conf/users.new")));
	    Enumeration e = users.keys ();
	    while (e.hasMoreElements ()) {
		String uname = (String)e.nextElement ();
		String pass = (String)users.get (uname);
		pw.println (uname + ":" + pass);
	    }
	    pw.flush ();
	    pw.close ();
	} catch (IOException e) {
	}
	if (error != null) 
	    Proxy.logError ("couldnt save accessconf");
	
    }

    /** Get the port this proxy is using.
     * @return the port number the proxy is listening on.
     */ 
    public static int getPort () {
	return port;
    }
    
    /** Get the local host.
     * @return the InetAddress of the host the proxy is running on.
     */
    public static InetAddress getHost () {
	return localhost;
    }

    /** return the configuration of the proxy.
     * @return the current configuration 
     */
    public static Properties getProperties () {
	return config;
    }

    /** simple test for now (until all pages are checked).
     */
    public static boolean isMetaRestricted () {
	return (Boolean.valueOf (config.getProperty ("restrictMeta")).booleanValue ());
    }
    
    /** returns the current connections.
     * @return a Vector with the current connections. 
     */
    public static Vector getCurrentConections () {
	return convec;
    }

    /** return the time when the proxy was started.
     * @return a Date with the time the proxy was started.
     */
    public static Date getStartDate () {
	return started;
    }
    
    /** return the number of pages that are fully served
     * @return The number of requests the proxy has handled
     */
    public static int getNoOfServedPages () {
	return total;
    }

    /** return the cache we have.
     * @return The Cache this proxy is using
     */
    public static NCache getCache () {
	return cache;
    }

    /** set up the filters to use.
     *  If a static setup is found in the class invoke it with the Properies for the section.
     */
    protected static void readFilterConfig () {
	filterVec = new Vector ();
	String filterconfig = config.getProperty ("filterconfig");
	if (filterconfig != null) {
	    try {
		Config cfg = new Config (filterconfig);
		String filters = cfg.getProperty ("", "filters");
		if (filters != null) {
		    StringTokenizer st = new StringTokenizer (filters, ",");
		    while (st.hasMoreElements ()) {
			String filterclass = (String)st.nextElement ();
			try {
			    Class cls = Class.forName (filterclass);  // throws exception unless found.
			    filterVec.addElement (cls);            // so this is safe.
	      
			    Class params[] = new Class[1];
			    params[0] = Properties.class;
			    Method setup = cls.getDeclaredMethod ("setup", params);
	      
			    Object arguments[] = new Object [1];
			    Properties pr = cfg.getProperties (filterclass);
			    if (pr == null)
				pr = new Properties ();
			    arguments[0] = pr;
			    setup.invoke (cls, arguments);	      
	      
			} catch (ClassNotFoundException e) {
			    Proxy.logError ("couldnt find filter: " + filterclass + " disabling it.");
			} catch (NoSuchMethodException e) {
			    Proxy.logError ("couldnt find method setup for: " + filterclass + " (this is probably nothing to worry about)");
			} catch (SecurityException e) {
			    Proxy.logError ("Seems reflection aint allowed.");	      
			} catch (InvocationTargetException e) {
			    Proxy.logError ("couldnt invoke setup for " + filterclass + 
					    ", nested exception is:" + e.getTargetException () + 
					    ". Setup didnt run correctly." + e);
			} catch (IllegalAccessException e) {
			    Proxy.logError (e.toString ());
			}
		    }
		}
	    } catch (IOException e) {
		Proxy.logError ("filterconfigfile (" + filterconfig +") not found, filtering disabled.");	
	    }
	}    
    }

    /** Get the Vector of allowed access
     * @return the Vector of access that are allowed.
     */
    public static Vector getAccesses () {
	return allowed;
    }

    /** reads in the accessfile and parses out the given ranges and types given.
     */
    protected static void readAccess () {
	String accessfile = config.getProperty ("access");
	if (accessfile != null) {
	    try {
		FileReader fr = new FileReader (accessfile);
		readAccess (fr);
		fr.close ();
	    } catch (FileNotFoundException e) {
		Proxy.logError ("Accessfile " + accessfile + " not found:" + e);
	    } catch (IOException e) {
		Proxy.logError ("Couldnt read accessfile:" + e);
	    }
	}
    }

    /** reads in the accessess allowed from the given Reader
     * @param r the Reader were data is available
     */
    public static void readAccess (Reader r) throws IOException {
	LineNumberReader br = new LineNumberReader (r);
	Vector allowed = new Vector ();
	String line = null;
	while ((line = br.readLine ()) != null) {
	    // remove comments....
	    int index = line.indexOf ('#');
	    if (index >= 0) 
		line = line.substring (0,index);
	    if (line.equals (""))
		continue;
	    StringTokenizer st = new StringTokenizer (line);
	    if (st.countTokens () != 3) {
		logError ("Bad line in accessconf:" + br.getLineNumber());
		continue;
	    }
	    String type = st.nextToken ();
	    String low = st.nextToken ();
	    InetAddress lowip = null;
	    try {
		lowip = InetAddress.getByName (low);	    
	    } catch (UnknownHostException e) {
		logError ("Bad host: " + low + " at line:" + br.getLineNumber());
		continue;	    
	    }
		
	    String high = st.nextToken ();
	    InetAddress highip = null;
	    try {
		highip = InetAddress.getByName (high);
	    } catch (UnknownHostException e) {
		logError ("Bad host: " + high + " at line:" + br.getLineNumber());
		continue;	    
	    }
	    
	    if (lowip != null && highip != null) 
		allowed.addElement (new Access (type, lowip, highip));
	}
	Proxy.allowed = allowed;
    }

    /** Get the Hashtable of users->password entries.
     * @return the Hashtable with the names and passwords of registered users.
     */
    public static Hashtable getUsers () {
	return users;
    }
    
    /** read in the users specified in basicusersfile
     */
    public static void readUsers () {
	String userfile = config.getProperty ("basicuserfile");
	if (userfile != null) {
	    try {
		FileReader br = new FileReader (userfile);
		readUsers (br);
		br.close ();
	    } catch (FileNotFoundException e) {
		Proxy.logError ("Userfile " + userfile + " not found:" + e);
	    } catch (IOException e) {
		Proxy.logError ("Couldnt read usersfile:" + e);
	    }
	}
    }

    /** reads the users from a given Reader 
     * @param r the Reader to read users from.
     */
    public static void readUsers (Reader r) throws IOException {
	BufferedReader br = new BufferedReader (r);
	Hashtable nusers = new Hashtable ();
	String line;
	while ((line = br.readLine ()) != null) {	  
	    StringTokenizer st = new StringTokenizer (line, ": \n\t");
	    String name = "";
	    String pass = "";
	    if (st.hasMoreTokens ())
		name = st.nextToken ();
	    else 
		continue;
	    if (st.hasMoreTokens ())
		pass = st.nextToken ();
	    nusers.put (name, pass);
	}
	users = nusers;
    }

    /** check if a user is authorised
     * @param userid the userid we check.
     * @param pass the password supplied.
     * @return true if user is in userlist, and has supplied a valid password, false otherwise.
     */
    public static boolean checkBasicAccess (String userid, String pass) {
	if (users == null)
	    return true;
	String realpass = (String)users.get (userid);
	boolean ret = realpass != null && realpass.equals (pass); 
	if (ret)
	    return ret;
	Proxy.logError ("Authentication failed for: " + userid);
	return false;
    }

    /** Get a standard (200 OK, text/html) header for the proxy, useful for metapages etc...
     * @return a HTTPHeader from this proxy.
     */
    public static HTTPHeader getHeader () {
	return getHeader ("HTTP/1.0 200 OK");
    }

    /** Get a header for the proxy. useful for metapages.
     * @param statusLine the Statusline of this header.
     * @return a HTTPHeader from this proxy.
     */
    public static HTTPHeader getHeader (String statusLine) {
	HTTPHeader ret = new HTTPHeader ();
	ret.setStatusLine (statusLine);
	ret.setHeader ("Server", Proxy.VERSION);
	ret.setHeader ("Content-type", "text/html");
	ret.setHeader ("Pragma", "no-cache");
	return ret;    
    }

    /** shutdown
     */
    protected static void kill () {
	accesslog.flush ();
	accesslog.close ();
	logError ("Killed!");
	errorlog.flush ();
	errorlog.close ();
	System.exit (0);
    }
}
