package prupp;

// $Id: Responder.java,v 1.9 1998/12/22 20:26:36 d94-rol Exp $

import java.io.*;
import java.net.*;

/** The base handler for a response from the server.
 */
public class Responder {
    /** outputstream to client */
    protected DataOutputStream clientos = null;
    /** inputstram from client */
    protected DataInputStream clientis = null;
    /** the requested url */
    protected String requestLine = null;
    /** the status of the page fetched (proxys status, not the response code from the web server) */
    protected int statusCode = 200;
    /** the authenticated userid if any */
    protected String authuser = "-";
    /** extra info we want */
    protected String extrainfo = "";
    
    /** basic constructor
     * @param reqLine The request being handled.
     * @param input The inputstream from the client.
     * @param output The outputstream to the client.
     */
    public Responder (String reqLine, DataInputStream input, DataOutputStream output) {
	clientos = output;
	clientis = input;
	requestLine = reqLine;
    }

    /** set the requestLine for this request 
     * @param reqLine the new requestLine
     */
    public void setRequestLine (String reqLine) {
	requestLine = reqLine;
    }

    /** get the requestLine for this request 
     * @return the requestLine
     */
    public String getRequestLine () {
	return requestLine;
    }
    
    /** sets the extra information we want to  save, this could be like compressionration etc
     * @param extrainfo the new information we want to save
     */
    public void setExtraInfo (String extrainfo) {
	this.extrainfo = extrainfo;
    }

    /** get the extrainfo for this request 
     * @return the extrainfo
     */
    public String getExtraInfo () {
	return extrainfo;
    }

    /** set the authuser for this request 
     * @param authuser the new authuser
     */
    public void setAuthuser (String authuser) {
	this.authuser = authuser;
    }

    /** get the authuser for this request 
     * @return the authuser
     */
    public String getAuthuser () {
	return authuser;
    }

    /** sets the statusCode.
     * @param statusCode the new StatusCode 
     */
    public void setStatusCode (int statusCode) {
	this.statusCode = statusCode;
    }    
    
    /** get the current status of this request
     * @return the statuscode for this request
     */
    public int getStatusCode () {
	return statusCode;
    }

    /** write a simple error page (400 Bad Request) to the client
     * @param message a description of the error
     */
    protected void error (String message) {
	setStatusCode (400);
	HTTPHeader header = Proxy.getHeader ("HTTP/1.0 400 Bad Request");
	StringBuffer error = new StringBuffer (header.toString () + 
					       HTMLPage.getPageHeader ("400 Bad Request") +
					       "Unable to handle request:<br><b>" + 
					       message +
					       "</b></body></html>\n");
	send (error.toString ());
    }

    /** write an illegal access (403) to the client
     * @param url the URL that is unaccessable.
     */
    protected void forbidden (URL url) {
	setStatusCode (403);
	HTTPHeader header = Proxy.getHeader ("HTTP/1.0 403 Forbidden");    
	StringBuffer res = new StringBuffer (header.toString () +
					     HTMLPage.getPageHeader ("403 Forbidden") +
					     "access to: <b>" + url + "</b> is forbidden\n" +
					     "</body></html>\n");
	send (res.toString ());
    }
  
    /** write a 407 Proxy Authentication Required to the client 
     * @param url the URL that is restricted
     */
    protected void requestAuthentication (String realm, URL url) {
	setStatusCode (407);
	HTTPHeader header = Proxy.getHeader ("HTTP/1.0 407 Proxy Authentication Required");    
	header.setHeader ("Proxy-Authenticate", "Basic realm=\"" + realm + "\"");
	StringBuffer accreq = new StringBuffer (header.toString () + 
						HTMLPage.getPageHeader ("407 Proxy Authentication Required") +
						"access to: <b>" + url + " </b><br>requires some authentication\n" +
						"</body></html>\n");
      
	send (accreq.toString());
    }
    
    /** Write a 500 Internal Server Error to the client (problem with the proxy).
     * @param message a description of the error that occured
     */  
    protected void internalError (String message) {
	setStatusCode (500);
	// in most cases we should have a header out already, but to be sure...
	HTTPHeader header = Proxy.getHeader ("HTTP/1.0 500 Internal Server Error ");  
	StringBuffer accreq = new StringBuffer (header.toString () + 
						HTMLPage.getPageHeader ("500 Internal Server Error") +
						"You have found a bug in RabbIT please report this(together with the URL you tried to visit) to the " +
						"<a href=\"http://www.nada.kth.se/projects/prup98/web_proxy/\" target = \"_top\">RabbIT</a> crew.<br>\n" +
						"error is:<BR><PRE>\n" + message + 
						"</PRE></body></html>\n");
	send (accreq.toString());
    }
    

    /** send data to the client
     * @param sb the StringBuffer holding the data to be sent
     */
    protected void send (String sb) {
	try {
	    if (clientos != null)
		clientos.write (sb.getBytes ());	
	} catch (IOException e) {
	    Proxy.logError ("couldnt write " + getStatusCode () + " to client:" + e);
	}
    }
}
