package prupp.filter;

// $Id$

import java.util.*;
import java.net.*;
import prupp.html.*;


/** add some text (and if given an url also an noproxy-link) directly after the body-tag.
 */
public class BodyFilter extends Filter
{
    /** the URL of the page we are filtering. */
    private URL url = null;
    /** there should be only one <bdoy...> tag in each page, we quit when its found */
    private boolean done = false;

    /** the identifier for pages filtered with this proxy. */
    private static final String prefix = "This page is filtered by RabbIT ";
    /** the string to append after our advertising. */
    private static final String postfix = "<br>";
    /** should we have a link to the unfiltered page? */
    private static final boolean unfilteredlink=false;
    
    private static Properties settings = new Properties ();
    
    /** default constructor
     */
    public BodyFilter () {
	this (null);
    }
    
    /** contruct with an URL to make the link correctly
     * @param url the URL of the page.
     */
    public BodyFilter (URL url) {
	this.url = url;
	setup (settings);
    }
    
    /** Add a identifier on top of the page.
     * @param block the HTMLBlock were filtering.
     * @return the uncertain area of the block.
     */
    public String filter(HTMLBlock block) {
	if (!done) {
	    Vector tags = block.getElements ();
	    for (int i = 0; i < tags.size (); i++) {
		Object o = tags.elementAt (i);
		if (!isString (o)) {
		    HTMLTag tag = (HTMLTag)o;
		    if (tag.getType ().equals ("BODY")) {
			done = true;
			int j = i + 1;
			tags.insertElementAt (settings.getProperty ("prefix"), j++);
			
			if (url != null && Boolean.valueOf (settings.getProperty ("unfilteredlink")).booleanValue ()) {
			    HTMLTag noproxy = new HTMLTag ("A");
			    noproxy.setAttribute ("HREF", url.getProtocol() + "://noproxy." + url.getHost () + 
						  ((url.getPort () > 0) ? ":" + url.getPort () : "") +
						  url.getFile () + 
						  ((url.getRef() != null) ? "#" + url.getRef () : ""));
			    tags.insertElementAt (noproxy, j++);
			    tags.insertElementAt ("unfiltered page", j++);
			    HTMLTag slasha = new HTMLTag ("/A");
			    tags.insertElementAt (slasha, j++);
			}
			tags.insertElementAt (settings.getProperty ("postfix"), j++);
			break;
		    }
		}
	    }
	}
	return null;
    }
    
    /** setup of this filter.
     * @param properties the properties to use.
     */
    public static void setup (Properties properties) {
	// keep old values if nothing specified.
	String pre = properties.getProperty ("prefix");
	if (pre == null) {
	    pre = settings.getProperty ("prefix");
	    if (pre == null)
		pre = prefix;
	}
	
	String post = properties.getProperty ("postfix");
	if (post == null) {
	    post = settings.getProperty ("postfix");
	    if (post == null)
		post = postfix;
	}
	
	// shut it off if not specified.
	String unfiltered = properties.getProperty ("unfilteredlink");
	if (unfiltered == null) {
	    unfiltered = settings.getProperty ("unfilteredlink");
	    if (unfiltered == null)
		unfiltered = "true";
	}

	settings.put ("prefix", pre);
	settings.put ("postfix", post);
	settings.put ("unfilteredlink", unfiltered);    
    }
    
    /** Called when you want to get the possible settings this filter 
     *  support. 
     */
    public static Properties getProperties () {
	return settings;
    }
}
