package prupp.html;

import java.util.*;

/** A class to represent a html-tag.
 */
public class HTMLTag 
{
    private String type = null;
    private Properties attributes = new Properties ();
    private Vector otherArgs = new Vector ();
    private boolean isComment = false;
    
    /** Create a tag with given type.
     * @param type the type of the tag e.g. A, IMG, B, P
     */
    public HTMLTag (String type) {
	this (type, false);
    }
    
    /** Create a tag with given type and choose if it is a comment.
     * @param type the type of the tag e.g. A, IMG, B, P
     * @param comment true if this tag is a comment false otherwise.
     */ 
    public HTMLTag (String type, boolean comment) {
	this.type = type;
	this.isComment = comment;
    }
    
    /** is this tag a comment?
     * @return true if this tag is a comment
     */
    public boolean isComment () {
	return isComment;
    }
    
    /** Get the attribute for the given type
     * @param type the key were looking for a value to.
     * @return the value, if any, or null.
     */
    public String getAttribute (String type) {
	return attributes.getProperty (type);
    }
    
    /** Set an attribute for this tag (like the NOWRAP attribute in TD-tags)
     * @param key the key were setting.
     */
    public void setAttribute (String key) {
	setAttribute (key, null);
    }

    /** Set a key=value parameter for this type
     * @param key the keystring to set.
     * @param value the value of the key, if null just set an attribute.
     */
    public void setAttribute (String key, String value) {
	if (value == null) {
	    otherArgs.addElement (key);
	} else {
	    attributes.put (key.toUpperCase (), value);
	}
    }
    
    /** removes the given attribute for the tag.
     * @param type the attribute to remove
     */
    public void removeAttribute (String type) {
	attributes.remove (type);
    }
    
    /** Give the string representation of this tag
     * @return a string describing this tag.
     */
    public String toString () {
	StringBuffer sb = new StringBuffer ("<");
	if (isComment ()) {
	    sb.append ("!-- ");
	    sb.append (type);
	}
	else {
	    sb.append (type);
	}
	for (int i = 0; i < otherArgs.size (); i++) {
	    String key = (String)otherArgs.elementAt (i);
	    sb.append (' ');
	    sb.append (key);
	}
	
	Enumeration keys = attributes.keys();
	while (keys.hasMoreElements ()) {
	    String key = (String)keys.nextElement ();
	    String val = attributes.getProperty (key);
	    sb.append (' ');
	    sb.append (key);
	    sb.append ("=\"");
	    sb.append (val);
	    sb.append ('"');
	}
	if (isComment ()) {
	    sb.append (" --");
	}
	sb.append ('>');
	
	return sb.toString ();
    }

    /** return the type of this tag.
     * @return the type of this tag.
     */
    public String getType () {
	return type;
    }
    
    /** set the type of this tag.
     * @param type the new type of this tag.
     */
    public void setType (String type) {
	this.type = type;
    }

    /** simple test program
     */
    public static void main (String args[]) {
	HTMLTag tag = new HTMLTag ("a");
	System.out.println ("tag: " + tag);
	tag.setAttribute ("href", "http://www.nada.kth.se/");
	System.out.println ("tag: " + tag);	
	tag.setAttribute ("target", "_top");
	tag.setAttribute ("NOWRAP");
	System.out.println ("tag: " + tag);	
    }
}



