package prupp.jconfig;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.util.*;

/** Applet for configuration of the RabbIT proxy.
 * All changes are written to disk when the user
 * clicks OK
 */
public class JConfig extends java.applet.Applet 
{
    Vector proxySettings = new Vector ();
    Vector proxyDescs = new Vector ();
    Vector proxyFields = new Vector ();
    TextField adReplacer;
    TextArea adLinks;
    TextArea access;
    TextArea users;

    Label status;
    Button ok;
    Button cancel;
    DataOutputStream os;
    BufferedReader is;
    Frame f;
    Panel panna;
    CardLayout card;

    /** Save one setting by sending a request to the proxy.
     * @param section proxy, acces, user or filter
     * @param setting Name of the setting (empty for users and access)
     * @param value New value to save
     */
    public void setOne (String section, String setting, String value) {
	String urls = "http://proxy/OneSetter?section=" +
	    section + "&setting=";
	
	if (!setting.equals (""))
	    value = setting + "=" + value;
	
	urls += URLEncoder.encode(value);
	
	try {
	    URLConnection urlc = new URL (urls).openConnection ();
	    urlc.setUseCaches (false);
	    is = new BufferedReader(new InputStreamReader(urlc.getInputStream ()));
	} catch (Exception e) {
	    System.err.println ("Ntt blev fel va!\n" + e);
	}
	
	try {
	    while (true) {
                String line = is.readLine ();
                if (line == null)
                    break;
                System.out.println (line);
            }
        } catch (IOException e) {
	    System.out.println ("Ooops.\n");
	    
        }

	try {
	    is.close ();
	} catch (IOException e) {
	    System.out.println ("Hrmpf. + " + e + "\n");
	}
    }

    /** Save all settings
     */
    public void updateSettings () {
	for (int i = 0; i < proxySettings.size (); i++)
	{
	    status.setText ("Saving \"" + 
			    proxySettings.elementAt (i).toString () + "\"...");

	    setOne ("proxy", proxySettings.elementAt (i).toString (), 	
		    ((TextField) proxyFields.elementAt (i)).getText ());
	}

	status.setText ("Saving AdFilter config...");
	setOne ("filter&filter=prupp.filter.AdFilter", "adreplacer", adReplacer.getText ());
	setOne ("filter&filter=prupp.filter.AdFilter", "adlinks", 
		adLinks.getText ().replace ('\n', ','));

	status.setText ("Saving user config...");
	setOne ("users", "", users.getText ());

	status.setText ("Saving access config...");
	setOne ("access", "", access.getText ());

	status.setText ("OK");
    }

    /** Load all settings from the proxy
     */
    public void getSettings (){
	for (int i = 0; i < proxySettings.size (); i++)
	{
	    status.setText ("Loading \"" + 
			    proxySettings.elementAt (i).toString () + "\"...");
	    
	    ((TextField) proxyFields.elementAt (i)).
		setText (getOne ("proxy", proxySettings.elementAt (i).toString ()));

	}
	
	status.setText ("Loading AdFilter settings...");
	adReplacer.setText (getOne ("filter&filter=prupp.filter.AdFilter", "adreplacer"));
	adLinks.setText (getOne ("filter&filter=prupp.filter.AdFilter", "adlinks").replace (',','\n'));

	status.setText ("Loading users...");
	users.setText (getOne ("users", ""));
	status.setText ("Loading access settings...");
	access.setText (getOne ("access", ""));
    }
    

    /** Load one setting from the proxy
     * @param section proxy, acces, user or filter
     * @param setting Name of the setting (empty for users and access)
     * @return Current value of the setting
     */
    public String getOne (String section, String setting) {
	String urls = "http://proxy/OneGetter?section=" +
	    section + "&setting=" + setting;
	String ret = "";
	String line;

	try {
	    URLConnection urlc = new URL (urls).openConnection ();
	    urlc.setUseCaches (false);
	    is = new BufferedReader(new InputStreamReader(urlc.getInputStream ()));

	    while ((line = is.readLine ()) != null) {
		ret += line + "\n";
	    }
	    is.close ();
	    
	    return ret.trim ();
	    
	}
	catch (Exception e) {
	    System.err.println (e + "\n");
	    return "";
	}
    }

    /** Used to set information about a control on the status line
     */
    public class StatusFocusListener extends FocusAdapter {
	String txt;
	
	/** Create a StatusFocusListener
	 *@param text The string to set status to
	 */
	StatusFocusListener (String text){
	    txt = text;
	}
	
	public void focusGained (FocusEvent e){
	    status.setText (txt);
	}
    }

    /** Used to change between the cards in the
     * CardLayout
     */
    public class Flipper implements ActionListener {
	String name;

	/* Create a Flipper
	 * @param name Name of the card to change to
	 */
	Flipper (String name){
	    this.name = name;
	}

	public void actionPerformed (ActionEvent e) {
	    card.show (panna, name);
	}	
    }
    
    /** Create the Section menu
     */
    public void buildMenu (Frame f) {
	    
	MenuBar mb = new MenuBar ();
	Menu sections = new Menu ("Sections");
	
	MenuItem mnuProxy = new MenuItem ("Proxy settings");
	mnuProxy.addActionListener (new Flipper ("Proxy"));
	MenuItem mnuAdFilter = new MenuItem ("Advertising filter");
	mnuAdFilter.addActionListener (new Flipper ("AdFilter"));
	MenuItem mnuAccess = new MenuItem ("Access list");
	mnuAccess.addActionListener (new Flipper ("Access"));
	MenuItem mnuUsers = new MenuItem ("Admin users");
	mnuUsers.addActionListener (new Flipper ("Users"));
	
	mb.add (sections);
	
	sections.add (mnuProxy);
	sections.add (mnuAdFilter);
	sections.add (mnuUsers);
	sections.add (mnuAccess);
	

	f.setMenuBar (mb);    
    }


    /** Create a new Frame and build a nice (hopefully) GUI
     */
    public void init ()
	{
	    StringTokenizer st;
	    f = new Frame ();

	    Panel p = new Panel ();
	    Panel p2 = new Panel ();
	    Panel p3 = new Panel ();

	    st = new StringTokenizer (getParameter("ProxySettings"),",");
	    while (st.hasMoreTokens ()) {
		proxySettings.addElement (st.nextToken ());
	    }
	    
	    st = new StringTokenizer (getParameter("ProxyDescs"),",");
	    while (st.hasMoreTokens ()) {
		proxyDescs.addElement (st.nextToken ());
	    }

	    ok = new Button ("OK");
	    ok.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent e) {
		    updateSettings ();
                }
            });

	    cancel = new Button ("Cancel");
	    cancel.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent e) {
		    f.setVisible (false);
		    f.dispose ();
                }
            });

	    p.setLayout(new GridLayout(0,4));

	    for (int i = 0; i < proxySettings.size (); i++)
	    {
		p.add (new Label (proxySettings.elementAt (i).toString ()));
		proxyFields.addElement (new TextField (12));
		p.add ((TextField) proxyFields.elementAt (i));
		((TextField) proxyFields.elementAt (i)).addFocusListener (
		    new StatusFocusListener (proxyDescs.elementAt (i).toString ()));
	    }
	    
	    p2.add (cancel);
	    p2.add (ok);
	    
	    p3.setLayout (new BorderLayout ());
	    status = new Label ("OK");
	    p3.add ("South", status);
	    p3.add ("North", p2);

	    Panel pAd = new Panel ();
	    pAd.setLayout (new BorderLayout ());
	    Panel p4 = new Panel ();
	    p4.setLayout (new BorderLayout ());
	    p4.add ("West", new Label ("adreplacer"));
	    adReplacer = new TextField (30);
	    adReplacer.addFocusListener (new StatusFocusListener (
		"Replace ads with this image"));
	    p4.add ("Center", adReplacer);
	    pAd.add ("North", p4);
	    
	    adLinks = new TextArea ("", 7, 20, TextArea.SCROLLBARS_VERTICAL_ONLY);
	    adLinks.addFocusListener (new StatusFocusListener (
		"Images that link to places containing these strings are ads"));
	    pAd.add ("Center", adLinks);

	    card = new CardLayout ();
	    panna = new Panel(card);

	    Panel p5 = new Panel (new BorderLayout ());
	    p5.add ("North", p);

	    Panel pAcc = new Panel (new BorderLayout ());
	    pAcc.add ("North", new Label ("Allow access from these IP ranges"));
	    access = new TextArea ("", 7, 20, TextArea.SCROLLBARS_VERTICAL_ONLY);
	    access.addFocusListener (new StatusFocusListener (
		"[ADMIN | HTTP]<blank><lower-ip><blank><upper-ip>"));
	    pAcc.add ("Center", access);

	    Panel pUsr = new Panel (new BorderLayout ());
	    pUsr.add ("North", new Label ("Users with administrative privileges"));
	    users = new TextArea ("", 7, 20, TextArea.SCROLLBARS_VERTICAL_ONLY);
	    users.addFocusListener (new StatusFocusListener (
		"One user per line, <name>:<password>"));
	    pUsr.add ("Center", users);


	    panna.add (p5, "Proxy");
	    panna.add (pAd, "AdFilter");
	    panna.add (pAcc, "Access");
	    panna.add (pUsr, "Users");
	    card.first (panna);

	    f.add ("Center", panna);
	    f.add ("South", p3);

	    f.addWindowListener (new WindowAdapter () {
                public void windowClosing (WindowEvent e) {
                    f.setVisible (false);
		    f.dispose ();
                }
            });

	    buildMenu (f);

	    status.setText ("OK");
	    f.setTitle ("Proxy Configuration");

	    f.validate ();
	    f.pack ();
	    f.show ();

	    setBackground (Color.white);
	    status.setText ("Loading settings...");
	    getSettings ();
	    status.setText ("OK");
	}

}



